@@ -1,5 +1,5 @@
 `PAMM` <-
-function (numsim, group, repl, randompart, fixed = c(0, 1, 0),intercept=0) 
+function (numsim, group, repl, randompart, fixed = c(0, 1, 0,-1,1),intercept=0,heteroscedasticity=c("null"),ftype="lmer") 
 {
     VI <- as.numeric(randompart[[1]])
     VS <- as.numeric(randompart[[2]])
@@ -18,9 +18,16 @@ function (numsim, group, repl, randompart, fixed = c(0, 1, 0),intercept=0)
         }
     }
     sigma <- matrix(c(VI, CovIS, CovIS, VS), ncol = 2)
+
+    Hetero <- heteroscedasticity[[1]]
+    het <- as.numeric(heteroscedasticity[-1])
+
     FM <- fixed[[1]]
     FV <- fixed[[2]]
     FE <- fixed[[3]]
+    if (length(fixed)==5) {
+    Xmin <- fixed[[4]]
+    Xmax <- fixed[[5]]}
     iD <- numeric(length(repl) * length(group))
     rp <- numeric(length(repl) * length(group))
     powersl <- numeric(numsim)
@@ -44,8 +51,14 @@ function (numsim, group, repl, randompart, fixed = c(0, 1, 0),intercept=0)
         for (r in repl) {
             N <- k * r
             for (i in 1:numsim) {
-                er <- rnorm(N, intercept, sqrt(VR))
+
                 EF <- rnorm(N, FM, sqrt(FV))
+                er <- numeric(length(N))
+                if (Hetero=="null") (er <- rnorm(N, intercept, sqrt(VR)))
+                if (Hetero=="power") (
+                for (n in 1:N) {er[n] <- rnorm(1, intercept, sqrt(VR*(het[1]+abs(EF[n])^het[2])^2))} )
+                if (Hetero=="exp")  (
+                for (n in 1:N) {er[n] <- rnorm(1, intercept, sqrt(VR*exp(2*het[1]*EF[n])))} )
                 db <- data.frame(ID = rep(1:k, r), obs = 1:N, 
                   error = er, EF = EF)
                 x <- rmvnorm(k, c(0, 0), sigma, method = "svd")
@@ -53,18 +66,34 @@ function (numsim, group, repl, randompart, fixed = c(0, 1, 0),intercept=0)
                 db$rand.sl <- rep(x[, 2], r)
                 db$Y <- db$rand.int + (db$rand.sl + FE) * db$EF + 
                   db$error
-                m.lm <- lm(Y ~ EF, data = db)
-                m1.lmer <- lmer(Y ~ EF + (1 | ID), data = db, 
-                  control = list(maxIter = 200, msMaxIter = 200))
-                pvint <- pchisq(-2 * (logLik(m.lm, REML = TRUE) - 
-                  logLik(m1.lmer, REML = TRUE))[[1]], 1, lower.tail = FALSE)
-                powerint[i] <- pvint <= 0.05
-                pvalint[i] <- pvint
-                m2.lmer <- lmer(Y ~ EF + (EF | ID), data = db, 
-                  control = list(maxIter = 200, msMaxIter = 200))
-                anosl <- anova(m2.lmer, m1.lmer)
-                powersl[i] <- anosl[2, "Pr(>Chisq)"] <= 0.05
-                pvalsl[i] <- anosl[2, "Pr(>Chisq)"]
+#                if (ftype=="lme") {
+#                m.lm <- lm(Y ~ EF, data = db)
+#                     m1.lme <- lme(Y ~ EF,random= ~1 | ID,weights=varConstPower(form=~EF), data = db) 
+#                 # control = list(maxIter = 200, msMaxIter = 200))
+#                     pvint <- pchisq(-2 * (logLik(m.lm, REML = TRUE) - 
+#                     logLik(m1.lme, REML = TRUE))[[1]], 1, lower.tail = FALSE)
+#                     powerint[i] <- pvint <= 0.05
+#                     pvalint[i] <- pvint
+#                     m2.lme <- lme(Y ~ EF,random= ~EF | ID,weights=varConstPower(form=~EF), data = db) 
+#                 # control = list(maxIter = 200, msMaxIter = 200))
+#                     anosl <- anova(m2.lme, m1.lme)
+ #                    powersl[i] <- anosl[2, "Pr(>Chisq)"] <= 0.05
+ #                    pvalsl[i] <- anosl[2, "Pr(>Chisq)"]               
+#                }
+#                else {
+                     m.lm <- lm(Y ~ EF, data = db)
+                     m1.lmer <- lmer(Y ~ EF + (1 | ID), data = db) 
+                 # control = list(maxIter = 200, msMaxIter = 200))
+                     pvint <- pchisq(-2 * (logLik(m.lm, REML = TRUE) - 
+                     logLik(m1.lmer, REML = TRUE))[[1]], 1, lower.tail = FALSE)
+                     powerint[i] <- pvint <= 0.05
+                     pvalint[i] <- pvint
+                     m2.lmer <- lmer(Y ~ EF + (EF | ID), data = db) 
+                 # control = list(maxIter = 200, msMaxIter = 200))
+                     anosl <- anova(m2.lmer, m1.lmer)
+                     powersl[i] <- anosl[2, "Pr(>Chisq)"] <= 0.05
+                     pvalsl[i] <- anosl[2, "Pr(>Chisq)"]
+#                 }
             }
             kk <- kk + 1
             iD[kk] <- k
